==> dart_icalendar-1.0.0.tar.gz: lib/models/icalendar_model.dart <==
import 'package:dart_icalendar/models/event_model.dart';

class ICalendar {
  final List<Event>? events;
  final String? prodId;
  final List<String>? properties;

  ICalendar(this.prodId, {this.events, this.properties});

  void addProperty(String property) {
    properties?.add(property);
  }

  String toICalString() {
    final buffer = StringBuffer();

    buffer.writeln('BEGIN:VCALENDAR' '\r');

    if (prodId != null) {
      buffer.writeln('PRODID:$prodId' '\r');
    }

    if (properties != null) {
      for (final property in properties ?? []) {
        buffer.writeln('$property' '\r');
      }
    }

    if (events != null) {
      for (final event in events ?? []) {
        buffer.write(event.toICalString());
      }
    }

    buffer.writeln('END:VCALENDAR' '\r');

    return buffer.toString();
  }

  static String _removeNonPrintable(String str) {
    return str.replaceAll(RegExp(r'[^\x20-\x7E]'), '');
  }

  static ICalendar fromICalString(String icalString) {
    final lines = icalString.split('\n');
    List<Event>? events;
    String? prodId;
    List<String>? properties;
    for (var i = 0; i < lines.length; i++) {
      lines[i] = _removeNonPrintable(lines[i]);
      final line = lines[i];
      if (line.startsWith('BEGIN:VEVENT')) {
        final eventLines = <String>[];
        while (!lines[i].startsWith('END:VEVENT')) {
          eventLines.add(lines[i]);
          i++;
        }
        events ??= [];
        events.add(Event.fromICalStrings(eventLines));
      } else if (line.startsWith('BEGIN:VCALENDAR') ||
          line.startsWith('END:VCALENDAR') ||
          line.isEmpty) {
        continue;
      } else if (line.startsWith('PRODID:')) {
        prodId = line.substring(7);
      } else {
        properties ??= [];
        properties.add(line);
      }
    }

    return ICalendar(
      prodId,
      events: events,
      properties: properties,
    );
  }

  static Event fromICalStrings(List<String> lines) {
    var uid = '';
    var summary = '';
    var description = '';
    DateTime? startDate;
    DateTime? endDate;

    for (var line in lines) {
      if (line.startsWith('UID:')) {
        uid = line.substring(4);
      } else if (line.startsWith('SUMMARY:')) {
        summary = line.substring(8);
      } else if (line.startsWith('DESCRIPTION:')) {
        description = line.substring(12);
      } else if (line.startsWith('DTSTART:')) {
        startDate = DateTime.parse(line.substring(8));
      } else if (line.startsWith('DTEND:')) {
        endDate = DateTime.parse(line.substring(6));
      }
    }

    return Event(
      uid: uid,
      summary: summary,
      description: description,
      startDate: startDate,
      endDate: endDate,
    );
  }

  void addEvent(Event event) {
    events?.add(event);
  }
}

==> dart_icalendar-1.0.1.tar.gz: lib/models/icalendar_model.dart <==
import 'package:dart_icalendar/models/event_model.dart';

class ICalendar {
  final List<Event>? events;
  final String? prodId;
  final List<String>? properties;

  ICalendar({this.prodId, this.events, this.properties});

  void addProperty(String property) {
    properties?.add(property);
  }

  String toICalString() {
    final buffer = StringBuffer();

    buffer.writeln('BEGIN:VCALENDAR' '\r');

    if (prodId != null) {
      buffer.writeln('PRODID:$prodId' '\r');
    }

    if (properties != null) {
      for (final property in properties ?? []) {
        buffer.writeln('$property' '\r');
      }
    }

    if (events != null) {
      for (final event in events ?? []) {
        buffer.write(event.toICalString());
      }
    }

    buffer.writeln('END:VCALENDAR' '\r');

    return buffer.toString();
  }

  static String _removeNonPrintable(String str) {
    return str.replaceAll(RegExp(r'[^\x20-\x7E]'), '');
  }

  static ICalendar fromICalString(String icalString) {
    final lines = icalString.split('\n');
    List<Event>? events;
    String? prodId;
    List<String>? properties;
    for (var i = 0; i < lines.length; i++) {
      lines[i] = _removeNonPrintable(lines[i]);
      final line = lines[i];
      if (line.startsWith('BEGIN:VEVENT')) {
        final eventLines = <String>[];
        while (!lines[i].startsWith('END:VEVENT')) {
          eventLines.add(lines[i]);
          i++;
        }
        events ??= [];
        events.add(Event.fromICalStrings(eventLines));
      } else if (line.startsWith('BEGIN:VCALENDAR') ||
          line.startsWith('END:VCALENDAR') ||
          line.isEmpty) {
        continue;
      } else if (line.startsWith('PRODID:')) {
        prodId = line.substring(7);
      } else {
        properties ??= [];
        properties.add(line);
      }
    }

    return ICalendar(
      prodId: prodId,
      events: events,
      properties: properties,
    );
  }

  static Event fromICalStrings(List<String> lines) {
    var uid = '';
    var summary = '';
    var description = '';
    DateTime? startDate;
    DateTime? endDate;

    for (var line in lines) {
      if (line.startsWith('UID:')) {
        uid = line.substring(4);
      } else if (line.startsWith('SUMMARY:')) {
        summary = line.substring(8);
      } else if (line.startsWith('DESCRIPTION:')) {
        description = line.substring(12);
      } else if (line.startsWith('DTSTART:')) {
        startDate = DateTime.parse(line.substring(8));
      } else if (line.startsWith('DTEND:')) {
        endDate = DateTime.parse(line.substring(6));
      }
    }

    return Event(
      uid: uid,
      summary: summary,
      description: description,
      startDate: startDate,
      endDate: endDate,
    );
  }

  void addEvent(Event event) {
    events?.add(event);
  }
}
